#!/bin/sh

#
# Z2 flash script v0.01 alpha 0
# Copyright (C) 2010 Marek Vasut <marek.vasut@gmail.com>
#
# This is under GPL v3
#
# Modified by mozzwald http://www.mozzwald.com

# Make sure only root can run our script
if [ "$(id -u)" != "0" ]; then
   echo "This script must be run as root" 1>&2
   exit 1
fi

echo "************************************************"
echo "           Zipit Z2 U-Boot flasher"
echo "************************************************"
echo ""
echo "WARNING: THIS COULD BRICK YOUR ZIPIT!"
echo "No one but YOU is responsible if something"
echo "goes wrong. "
echo ""

image=image.img
partlist=plist.lst
flashdefs=flash.def

echo "Preparing flash image.."
if [ -e "$image" ] ; then
	rm -f "$image";
fi
touch "$image"

if [ -e "$partlist" ] ; then
	rm -f "$partlist";
fi
touch "$partlist"

if [ -e "restore.def" ]; then
	rm -f restore.def
fi
touch restore.def

# create the complete image file with proper spacing
offset=0
while read file start md5; do
	if ! [ -e "$file" ] ; then
		echo "File \"$file\" not found, aborting!"
		exit 1;
	fi

	# check md5sum
	if [ -z "$md5" ] ; then
		echo "md5sum for \"$file\" not found, aborting!"
		exit 2;
	elif [ "$md5" != "`/mnt/sd0/bin/busybox md5sum $file | /mnt/sd0/bin/busybox cut -d " " -f 1`" ] ; then
		echo "md5sum for \"$file\" does not match, aborting!"
		exit 3;
	fi

	# calculate empty space between parts
	pad=`expr "$start" - "$offset"`

	if [ "$pad" -lt 0 ] ; then
		echo "Image components overlap, aborting!"
		exit 2;
	elif [ "$pad" -gt 0 ] ; then
		# write empty space to image file
		dd if=/dev/zero bs="$pad" count=1 >> "$image" 2>/dev/null;
	fi

	# write part to image file
	cat "$file" >> "$image"
	# calculate next offset
	offset=`wc -c "$image" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`;
	#echo "Offset: $offset"
	md5="\0"
done < "$flashdefs"
echo "$image ready for flashing."

# fix mtdblock partitions and create partition list
#TODO: Make backup and the restore.def file here
flashsz=0
for i in `/mnt/sd0/bin/busybox ls -1 /dev/ | /mnt/sd0/bin/busybox egrep "mtdblock[[:digit:]]+"` ; do
	# fix the mtdblock
	echo "Fixing /dev/$i"
	blocknum=`echo "$i" | sed -e "s/^.*\(.\)$/\1/"`
	rm "/dev/$i"
	mknod "/dev/$i" b 31 "$blocknum"
	# write the partition list
	partsz=`/mnt/sd0/bin/busybox wc -c "/dev/$i" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
	echo "/dev/$i $partsz" >> "$partlist"
	# Backup Partitions & Create restore.def
	echo "Backing up /dev/$i"
        if [ -e "$i.bak" ]; then
                rm -f "$i.bak"
        fi
	dd if="/dev/$i" of="$i.bak"
        mtdmd5=`/mnt/sd0/bin/busybox md5sum "$i.bak" | /mnt/sd0/bin/busybox cut -d " " -f 1`
        echo "$i.bak $flashsz $mtdmd5" >> restore.def
	flashsz=`expr "$flashsz" + "$partsz"`
	mtdcount=`expr "$blocknum" + 1`
	echo "$blocknum PartSize: $partsz FlashSize: $flashsz"
done

if [ "$offset" -gt "$flashsz" ] ; then
	echo "Image too big for flash, aborting!"
	echo "Offset: $offset FlashSize: $flashsz"
	exit 3;
fi

if [ -z "$partlist" ] ; then
	echo "Partition list empty, aborting!"
	exit 4;
fi

# Backup partitions
#count=0
#while read part size ; do
#        echo "Backing up $part"
#	mtdfile=mtdblock"$count".bak
#	if [ -e "$mtdfile" ]; then
#		rm -f "$mtdfile"
#	fi
#       dd if="$part" of="$mtdfile"
	# Prepare & add line to the restore definition file
#	if [ "$count" -eq 0 ]; then
#		roffset=0
#	else
#		prevcount=`expr "$count" - 1`
#		prevmtd=mtdblock"$prevcount".bak
#		roffset=`/mnt/sd0/bin/busybox wc -c "$prevmtd"`
#	fi
#	mtdmd5=`/mnt/sd0/bin/busybox md5sum "$mtdfile" | /mnt/sd0/bin/busybox cut -d " " -f 1`
#	echo "$mtdfile 0 $mtdmd5" >> restore.def
#        count=`expr "$count" + 1`
#done < "$partlist"

echo "Make sure your Zipit is fully charged and"
echo "you have the AC Adapter plugged in. "

agree=" "
read -p "Last chance before flashing. Are you sure? (yes/no): " agree
if [ "$agree" != "yes" ]; then
  echo "You chose not to continue."
  echo "Did you type \"yes\" correctly?"
  echo "exiting..."
  exit 5
fi

echo "Partions to be over-written: $mtdcount"
echo "Starting flashing procedure, do NOT POWER OFF!"

offset=0
while read part size ; do
	echo "Writing to $part"
	echo dd if="$image" of="$part" bs=1 skip="$offset" count="$size"
	offset=`expr "$offset" + "$size"`;
done < "$partlist"

rm -f "$partlist" "$image"

echo "Flashing complete!"
agree=0
read -p "Are you ready to halt the system? (yes/no): " agree
if [ "$agree" != "yes" ]; then
  echo "When you are finished with the shell, be sure to"
  echo "unplug the AC Power and remove the battery for a"
  echo "moment."
  echo ""
  echo "Re-attach the battery/AC and the system should boot."
  echo "Exiting to shell..."
  exit 0
fi

echo "After the system is halted, unplug the AC Power"
echo "and remove the battery for a moment. After you"
echo "re-attach the battery/AC, the system should boot."
echo "Shutting down the system..."
sleep 5
halt
